/*
 *  Copyright 2019-2025 Zheng Jie
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package com.fuyuanshen.modules.system.rest;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import com.fuyuanshen.annotation.Log;
import com.fuyuanshen.exception.BadRequestException;
import com.fuyuanshen.modules.security.service.UserCacheManager;
import com.fuyuanshen.modules.security.service.dto.JwtUserDto;
import com.fuyuanshen.modules.system.constant.UserConstants;
import com.fuyuanshen.modules.system.domain.Dept;
import com.fuyuanshen.modules.system.domain.Role;
import com.fuyuanshen.modules.system.domain.User;
import com.fuyuanshen.modules.system.domain.dto.UserPassVo;
import com.fuyuanshen.modules.system.domain.dto.UserQueryCriteria;
import com.fuyuanshen.modules.system.domain.vo.ConsumerVo;
import com.fuyuanshen.modules.system.mapper.UserMapper;
import com.fuyuanshen.modules.system.service.*;
import com.fuyuanshen.modules.utils.ResponseVO;
import com.fuyuanshen.utils.PageResult;
import com.fuyuanshen.utils.SecurityUtils;
import com.fuyuanshen.utils.StringUtils;
import com.fuyuanshen.utils.enums.CodeEnum;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author Zheng Jie
 * @date 2018-11-23
 */
@Slf4j
@Api(tags = "系统：用户管理")
@RestController
@RequestMapping("/api/users")
@RequiredArgsConstructor
public class UserController {

    private final String PASSWORDRSAKEY = "e10adc3949ba59abbe56e057f20f883e";

    private final PasswordEncoder passwordEncoder;
    private final UserService userService;
    private final DataService dataService;
    private final DeptService deptService;
    private final RoleService roleService;
    private final VerifyService verificationCodeService;
    private final UserCacheManager userCacheManager;
    private final UserMapper userMapper;


    @ApiOperation("导出用户数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('user:list')")
    public void exportUser(HttpServletResponse response, UserQueryCriteria criteria) throws IOException {
        userService.download(userService.queryAll(criteria), response);
    }


    @ApiOperation("查询用户")
    @GetMapping
    @PreAuthorize("@el.check('user:list')")
    public ResponseVO<PageResult<User>> queryUser(UserQueryCriteria criteria) {
        Page<Object> page = new Page<>(criteria.getPage(), criteria.getSize());
        if (!ObjectUtils.isEmpty(criteria.getDeptId())) {
            criteria.getDeptIds().add(criteria.getDeptId());
            // 先查找是否存在子节点
            List<Dept> data = deptService.findByPid(criteria.getDeptId());
            // 然后把子节点的ID都加入到集合中
            criteria.getDeptIds().addAll(deptService.getDeptChildren(data));
        }
        // 数据权限
        List<Long> dataScopes = dataService.getDeptIds(userService.findByName(SecurityUtils.getCurrentUsername()));
        // criteria.getDeptIds() 不为空并且数据权限不为空则取交集
        if (!CollectionUtils.isEmpty(criteria.getDeptIds()) && !CollectionUtils.isEmpty(dataScopes)) {
            // 取交集
            criteria.getDeptIds().retainAll(dataScopes);
            if (!CollectionUtil.isEmpty(criteria.getDeptIds())) {
                return ResponseVO.success(userService.queryAll(criteria, page));
            }
        } else {
            // 否则取并集
            criteria.getDeptIds().addAll(dataScopes);
            return ResponseVO.success(userService.queryAll(criteria, page));
        }
//        return new ResponseEntity<>(PageUtil.noData(),HttpStatus.OK);
        return ResponseVO.success(null);
    }


    /**
     * 分页查询客户(限制子客户)
     *
     * @param criteria
     * @return
     */
    @ApiOperation("分页查询客户")
    @GetMapping(value = "/customer")
    public ResponseVO<PageResult<ConsumerVo>> queryCustomer(UserQueryCriteria criteria) {
        User user = userMapper.findByUsername(SecurityUtils.getCurrentUsername());
        Page<User> page = new Page<>(criteria.getPage(), criteria.getSize());
        // JwtUserDto userCache = userCacheManager.getUserCache(SecurityUtils.getCurrentUsername());
        // 声明在外部保证作用域
        Set<Long> userIds = new HashSet<>();
        // 非超级管理员增加数据隔离
        if (user.getTenantId() != null && !user.getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
            // Long currentUserId = SecurityUtils.getCurrentUserId();
            userIds.add(user.getId());
            // 声明在外部保证作用域
            // if (CollectionUtil.isNotEmpty(subUsers)) {
            //     subUserIds = subUsers.stream().map(User::getId).collect(Collectors.toSet());
            // } else {
            //     // 创建包含默认值0的集合
            //     subUserIds = Collections.singleton(0L);
            // }
            // subUserIds.add(user.getId());
            // 统一设置ID集合
            // criteria.setIds(subUserIds);
        }
        // 统一设置ID集合
        criteria.setIds(userIds);
        PageResult<ConsumerVo> pageUsers = userService.queryCustomers(criteria, page);

        return ResponseVO.success(pageUsers);
    }


    /**
     * 分页查询客户（树结构）
     *
     * @param criteria
     * @return
     */
    @ApiOperation("分页查询客户（树结构）")
    @GetMapping(value = "/customerTree")
    public ResponseVO<PageResult<ConsumerVo>> queryCustomerTree(UserQueryCriteria criteria) {
        User user = userMapper.findByUsername(SecurityUtils.getCurrentUsername());
        Page<User> page = new Page<>(criteria.getPage(), criteria.getSize());
        // JwtUserDto userCache = userCacheManager.getUserCache(SecurityUtils.getCurrentUsername());
        // 声明在外部保证作用域
        // Set<Long> subUserIds = new HashSet<>();
        // 非超级管理员增加数据隔离
        // if (user.getTenantId() != null && !user.getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
        //     Long currentUserId = SecurityUtils.getCurrentUserId();
        //     List<User> subUsers = userService.findUserTree(user.getId());
        //     // 声明在外部保证作用域
        //     Set<Long> subUserIds = new HashSet<>();
        //     if (CollectionUtil.isNotEmpty(subUsers)) {
        //         subUserIds = subUsers.stream().map(User::getId).collect(Collectors.toSet());
        //     } else {
        //         // 创建包含默认值0的集合
        //         subUserIds = Collections.singleton(0L);
        //     }
        //     subUserIds.add(user.getId());
        //     // 统一设置ID集合
        //     criteria.setIds(subUserIds);
        // }
        // // 统一设置ID集合
        // // criteria.setIds(subUserIds);
        // PageResult<ConsumerVo> pageUsers = userService.queryCustomers(criteria, page);

        // 声明在外部保证作用域
        Set<Long> userIds = new HashSet<>();
        // 非超级管理员增加数据隔离
        if (user.getTenantId() != null && !user.getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
            userIds.add(user.getId());
        }
        // 统一设置ID集合
        criteria.setIds(userIds);
        PageResult<ConsumerVo> pageUsers = userService.queryCustomers(criteria, page);

        return ResponseVO.success(pageUsers);
    }


    @GetMapping(value = "/allCustomer")
    @ApiOperation("查询所有客户")
    public ResponseVO<List<User>> queryAllCustomer(UserQueryCriteria criteria) {
        User user = userMapper.findByUsername(SecurityUtils.getCurrentUsername());
        // 声明在外部保证作用域
        Set<Long> userIds = new HashSet<>();
        // 非超级管理员增加数据隔离
        if (user.getTenantId() != null && !user.getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
            userIds.add(user.getId());
        }
        // 统一设置ID集合
        criteria.setIds(userIds);

        List<User> users = userService.queryAllCustomers(criteria);
        return ResponseVO.success(users);
    }


    @Log("新增用户")
    @ApiOperation("新增用户")
    @PostMapping
    @PreAuthorize("@el.check('user:add')")
    public ResponseVO<Object> createUser(@Validated @RequestBody User resources) {
        checkLevel(resources);
        // 默认密码 123456
        // resources.setPassword(passwordEncoder.encode("123456"));
        resources.setPassword(PASSWORDRSAKEY);
        userService.create(resources);
        return ResponseVO.success(null);
    }


    @Log("新增客户")
    @ApiOperation("新增客户")
    @PostMapping(value = "/addCustomer")
    public ResponseVO<Object> addCustomer(@Validated @RequestBody User user) {
        if (StringUtils.isBlank(user.getPassword())) {
            throw new BadRequestException("账号密码不能为空");
        }
        checkLevel(user);
        userService.addCustomer(user);
        JwtUserDto jwtUser = (JwtUserDto) SecurityUtils.getCurrentUser();
        return ResponseVO.success("新增客户成功！！！");
    }


    @Log("修改用户")
    @ApiOperation("修改用户")
    @PutMapping
    @PreAuthorize("@el.check('user:edit')")
    public ResponseVO<Object> updateUser(@Validated(User.Update.class) @RequestBody User resources) throws Exception {
        checkLevel(resources);
        userService.update(resources);
        return ResponseVO.success(null);
    }


    @Log("修改客户")
    @ApiOperation("修改客户")
    @PutMapping(value = "updateCustomer")
    public ResponseVO<Object> updateCustomer(@RequestBody User resources) throws Exception {
        if (resources.getId() == null) {
            throw new BadRequestException("客户ID不能为空");
        }
        checkLevel(resources);
        userService.updateCustomer(resources);
        return ResponseVO.success(null);
    }

    @Log("修改用户：个人中心")
    @ApiOperation("修改用户：个人中心")
    @PutMapping(value = "center")
    public ResponseVO<Object> centerUser(@Validated(User.Update.class) @RequestBody User resources) {
        if (!resources.getId().equals(SecurityUtils.getCurrentUserId())) {
            throw new BadRequestException("不能修改他人资料");
        }
        userService.updateCenter(resources);
        return ResponseVO.success(null);
    }


    @Log("删除用户")
    @ApiOperation("删除用户")
    @DeleteMapping
    @PreAuthorize("@el.check('user:del')")
    public ResponseVO<Object> deleteUser(@RequestBody Set<Long> ids) {

        for (Long id : ids) {
            if (id.equals(UserConstants.SUPER_ADMIN_ID)) {
                throw new BadRequestException("不能删除超级管理员用户");
            }
            List<Role> roleList = roleService.findByUsersId(id);
            if (CollectionUtil.isNotEmpty(roleList)) {
                Integer currentLevel = Collections.min(roleService.findByUsersId(SecurityUtils.getCurrentUserId()).stream().map(Role::getLevel).collect(Collectors.toList()));
                Integer optLevel = Collections.min(roleList.stream().map(Role::getLevel).collect(Collectors.toList()));
                if (currentLevel > optLevel) {
                    throw new BadRequestException("角色权限不足，不能删除：" + userService.findById(id).getUsername());
                }
            }
        }
        userService.delete(ids);
        ids.remove(SecurityUtils.getCurrentUserId());
        return ResponseVO.success(null);
    }


    @Log("删除客户")
    @ApiOperation("删除客户")
    @DeleteMapping(value = "/deleteCustomer")
    public ResponseVO<Object> deleteCustomer(@RequestBody Set<Long> ids) {
        if (CollectionUtil.isEmpty(ids)) {
            throw new BadRequestException("请选择要删除的客户");
        }
        /*for (Long id : ids) {
            Integer currentLevel =  Collections.min(roleService.findByUsersId(SecurityUtils.getCurrentUserId()).stream().map(Role::getLevel).collect(Collectors.toList()));
            Integer optLevel =  Collections.min(roleService.findByUsersId(id).stream().map(Role::getLevel).collect(Collectors.toList()));
            if (currentLevel > optLevel) {
                throw new BadRequestException("角色权限不足，不能删除：" + userService.findById(id).getUsername());
            }
        }*/
        userService.delete(ids);
        return ResponseVO.success(null);
    }


    @ApiOperation("修改密码")
    @PostMapping(value = "/updatePass")
    public ResponseVO<Object> updateUserPass(@RequestBody UserPassVo passVo) throws Exception {
        // String oldPass = RsaUtils.decryptByPrivateKey(RsaProperties.privateKey, passVo.getOldPass());
        // String newPass = RsaUtils.decryptByPrivateKey(RsaProperties.privateKey, passVo.getNewPass());
        User user = userService.findByName(SecurityUtils.getCurrentUsername());
        // if (!passwordEncoder.matches(passVo.getOldPass(), user.getPassword())) {
        //     throw new BadRequestException("修改失败，旧密码错误");
        // }
        // if (passwordEncoder.matches(passVo.getNewPass(), user.getPassword())) {
        //     throw new BadRequestException("新密码不能与旧密码相同");
        // }

        if (!user.getPassword().equals(passVo.getOldPass())) {
            throw new BadRequestException("修改失败，旧密码错误");
        }
        if (user.getPassword().equals(passVo.getNewPass())) {
            throw new BadRequestException("新密码不能与旧密码相同");
        }

        // userService.updatePass(user.getUsername(), passwordEncoder.encode(newPass));
        userService.updatePass(user.getUsername(), passVo.getNewPass());
        return ResponseVO.success(null);
    }


    @ApiOperation("重置密码")
    @PutMapping(value = "/resetPwd")
    public ResponseVO<Object> resetPwd(@RequestBody Set<Long> ids) {
        String pwd = passwordEncoder.encode("123456");
        userService.resetPwd(ids, pwd);
        return ResponseVO.success(null);
    }

    @ApiOperation("修改头像")
    @PostMapping(value = "/updateAvatar")
    public ResponseVO<Object> updateUserAvatar(@RequestParam MultipartFile avatar) {
//        return new ResponseEntity<>(userService.updateAvatar(avatar), HttpStatus.OK);
        return ResponseVO.success(userService.updateAvatar(avatar));
    }

    @Log("修改邮箱")
    @ApiOperation("修改邮箱")
    @PostMapping(value = "/updateEmail/{code}")
    public ResponseVO<Object> updateUserEmail(@PathVariable String code, @RequestBody User resources) throws Exception {
        // String password = RsaUtils.decryptByPrivateKey(RsaProperties.privateKey, resources.getPassword());
        User user = userService.findByName(SecurityUtils.getCurrentUsername());
        if (!passwordEncoder.matches(resources.getPassword(), user.getPassword())) {
            throw new BadRequestException("密码错误");
        }
        verificationCodeService.validated(CodeEnum.EMAIL_RESET_EMAIL_CODE.getKey() + resources.getEmail(), code);
        userService.updateEmail(user.getUsername(), resources.getEmail());
        return ResponseVO.success(null);
    }

    /**
     * 如果当前用户的角色级别低于创建用户的角色级别，则抛出权限不足的错误
     *
     * @param resources /
     */
    private void checkLevel(User resources) {
        Integer currentLevel = Collections.min(roleService.findByUsersId(SecurityUtils.getCurrentUserId()).stream().map(Role::getLevel).collect(Collectors.toList()));
        Integer optLevel = roleService.findByRoles(resources.getRoles());
        if (currentLevel > optLevel) {
            throw new BadRequestException("角色权限不足");
        }
    }

}
